import typing as t

def mangle_nbsp(
    html: str,
    STRIP_WHITESPACE_NBSP_MANGLE_CHARACTER: t.Optional[str],
) -> str:
    html = html.replace("&nbsp;", STRIP_WHITESPACE_NBSP_MANGLE_CHARACTER)
 
    return html


def unmangle_nbsp(
    html: str,
    STRIP_WHITESPACE_NBSP_MANGLE_CHARACTER: t.Optional[str],
) -> str:
    html = html.replace(STRIP_WHITESPACE_NBSP_MANGLE_CHARACTER, "&nbsp;")
    
    return html



# Minifiers
import typing as t

# 3rd party
from minify_html import minify as rust_minifier

from .html import (
    html_minify as python_minifier,
    mangle_nbsp,
    unmangle_nbsp,
)

# Import helper functions
from .html import add_line_break


def minify(
    buffer: str,
    # Rust
    STRIP_WHITESPACE_RUST_DO_NOT_MINIFY_DOCTYPE: t.Optional[bool] = True,
    STRIP_WHITESPACE_RUST_ENSURE_SPEC_CONPLIANT_UNQUOTED_ATTRIBUTE_VALUES: t.Optional[
        bool
    ] = True,
    STRIP_WHITESPACE_RUST_KEEP_CLOSING_TAGS: t.Optional[bool] = True,
    STRIP_WHITESPACE_RUST_KEEP_COMMENTS: t.Optional[bool] = True,
    STRIP_WHITESPACE_RUST_KEEP_HTML_AND_HEAD_OPENING_TAGS: t.Optional[bool] = True,
    STRIP_WHITESPACE_RUST_KEEP_SPACES_BETWEEN_ATTRIBUTES: t.Optional[bool] = True,
    STRIP_WHITESPACE_RUST_MINIFY_CSS: t.Optional[bool] = True,
    STRIP_WHITESPACE_RUST_MINIFY_JS: t.Optional[bool] = True,
    STRIP_WHITESPACE_RUST_REMOVE_BANGS: t.Optional[bool] = True,
    STRIP_WHITESPACE_RUST_REMOVE_PROCESSING_INSTRUCTIONS: t.Optional[bool] = True,
    # Python
    STRIP_WHITESPACE_PYTHON_REMOVE_COMMENTS: t.Optional[bool] = False,
    STRIP_WHITESPACE_PYTHON_CONDENSE_STYLE_FROM_HTML: t.Optional[bool] = True,
    STRIP_WHITESPACE_PYTHON_CONDENSE_SCRIPT_FROM_HTML: t.Optional[bool] = True,
    STRIP_WHITESPACE_PYTHON_CLEAN_UNNEEDED_HTML_TAGS: t.Optional[bool] = True,
    STRIP_WHITESPACE_PYTHON_CONDENSE_HTML_WHITESPACE: t.Optional[bool] = True,
    STRIP_WHITESPACE_PYTHON_UNQUOTE_HTML_ATTRIBUTES: t.Optional[bool] = True,
    # NBSP character setting
    STRIP_WHITESPACE_NBSP_MANGLE_CHARACTER: t.Optional[str] = "'অ'",
    # Compression Settings
    STRIP_WHITESPACE_REGEX_FLAVOR: t.Optional[str] = str(
        "alpinejs"  # Lets default it to alpinejs.
    ),
) -> bytes:
    #   Errors ⚠️
    #   Checked here ✔️ | ❌

    # ❌ STRIP_WHITESPACE_REGEX_FLAVOR is not defined in module
    if str(STRIP_WHITESPACE_REGEX_FLAVOR) not in [
        str("plain"),
        str("alpinejs"),
        str("petitevue"),
    ]:
        raise ValueError(
            f"""Error in python_strip_whitespace.compress

                STRIP_WHITESPACE_REGEX_FLAVOR is not defined.
                Please change the value when calling minify function.

                    Current Value : { str(STRIP_WHITESPACE_REGEX_FLAVOR) }
                    It must be one of these :
                        |>  str("plain")
                        |>  str("alpinejs")
                        |>  str("petitevue")"""
        )

    # Declare some variables here

    #   First change the &nbsp; into a special character so the other compressors cant mangle with that.
    first_iter: str = mangle_nbsp(
        buffer,
        STRIP_WHITESPACE_NBSP_MANGLE_CHARACTER,
    )

    #   Rust based minifier. The most powerful one in here. 💪
    second_iter: str = rust_minifier(
        first_iter,
        do_not_minify_doctype=STRIP_WHITESPACE_RUST_DO_NOT_MINIFY_DOCTYPE,
        ensure_spec_compliant_unquoted_attribute_values=STRIP_WHITESPACE_RUST_ENSURE_SPEC_CONPLIANT_UNQUOTED_ATTRIBUTE_VALUES,
        keep_closing_tags=STRIP_WHITESPACE_RUST_KEEP_CLOSING_TAGS,
        keep_comments=STRIP_WHITESPACE_RUST_KEEP_COMMENTS,
        keep_html_and_head_opening_tags=STRIP_WHITESPACE_RUST_KEEP_HTML_AND_HEAD_OPENING_TAGS,
        keep_spaces_between_attributes=STRIP_WHITESPACE_RUST_KEEP_SPACES_BETWEEN_ATTRIBUTES,
        minify_css=STRIP_WHITESPACE_RUST_MINIFY_CSS,
        minify_js=STRIP_WHITESPACE_RUST_MINIFY_JS,
        remove_bangs=STRIP_WHITESPACE_RUST_REMOVE_BANGS,
        remove_processing_instructions=STRIP_WHITESPACE_RUST_REMOVE_PROCESSING_INSTRUCTIONS,
    )

    #   Rust minifier comes first to migrate some of the issues I faced.😛
    #   Specially the python module picks '\n in class=""
    #   So first remove all unnecessary whitespace before adding line_break
    third_iter: str = add_line_break(second_iter, STRIP_WHITESPACE_REGEX_FLAVOR)

    #   Finally the python iterator.
    #   I don't know how this works.🤷
    #   So adding it at last
    fourth_iter: str = python_minifier(
        third_iter,
        STRIP_WHITESPACE_PYTHON_REMOVE_COMMENTS,
        STRIP_WHITESPACE_PYTHON_CONDENSE_STYLE_FROM_HTML,
        STRIP_WHITESPACE_PYTHON_CONDENSE_SCRIPT_FROM_HTML,
        STRIP_WHITESPACE_PYTHON_CLEAN_UNNEEDED_HTML_TAGS,
        STRIP_WHITESPACE_PYTHON_CONDENSE_HTML_WHITESPACE,
        STRIP_WHITESPACE_PYTHON_UNQUOTE_HTML_ATTRIBUTES,
    )

    #   Replace special character with &nbsp;
    return_buffer: bytes = unmangle_nbsp(
        fourth_iter,
        STRIP_WHITESPACE_NBSP_MANGLE_CHARACTER,
    ).encode()

    return return_buffer



from re import compile
from typing import Pattern

# X Pattern

X_INIT_PATTERN: Pattern = compile(r"(?<=x-init=[\"\`])([^[\"\`\]]+)")
X_EFFECT_PATTERN: Pattern = compile(r"(?<=x-effect=[\"\`])([^[\"\`\]]+)")

# Click Related Pattern.
# Mapped from => https://alpinejs.dev/directives/on#window

CLICK_PATTERN: Pattern = compile(r"(?<=click=[\"\`])([^[\"\`\]]+)")
CLICK_PREVENT_PATTERN: Pattern = compile(r"(?<=@click.prevent=[\"\`])([^[\"\`\]]+)")
CLICK_STOP_PATTERN: Pattern = compile(r"(?<=@click.stop=[\"\`])([^[\"\`\]]+)")
CLICK_OUTSIDE_PATTERN: Pattern = compile(r"(?<=@click.outside=[\"\`])([^[\"\`\]]+)")
CLICK_WINDOW_PATTERN: Pattern = compile(r"(?<=@click.window=[\"\`])([^[\"\`\]]+)")
CLICK_ONCE_PATTERN: Pattern = compile(r"(?<=@click.document=[\"\`])([^[\"\`\]]+)")
CLICK_DEBOUNCE_PATTERN: Pattern = compile(r"(?<=@click.debounce=[\"\`])([^[\"\`\]]+)")
CLICK_THROTTLE_PATTERN: Pattern = compile(r"(?<=@click.throttle=[\"\`])([^[\"\`\]]+)")
CLICK_SELF_PATTERN: Pattern = compile(r"(?<=@click.self=[\"\`])([^[\"\`\]]+)")
CLICK_CAMEL_PATTERN: Pattern = compile(r"(?<=@click.camel=[\"\`])([^[\"\`\]]+)")
CLICK_DOT_PATTERN: Pattern = compile(r"(?<=@click.dot=[\"\`])([^[\"\`\]]+)")
CLICK_PASSIVE_PATTERN: Pattern = compile(r"(?<=@click.passive=[\"\`])([^[\"\`\]]+)")


# Mouse movement Pattern
# Mapped from => https://www.w3schools.com/jsref/obj_mouseevent.asp

MOUSE_ENTER_PATTERN: Pattern = compile(r"(?<=mouseenter=[\"\`])([^[\"\`\]]+)")
MOUSE_LEAVE_PATTERN: Pattern = compile(r"(?<=mouseleave=[\"\`])([^[\"\`\]]+)")
# MOUSE_CONTEXT_MENU_PATTERN: Pattern = compile(r"(?<=mousecontextmenu=[\"\`])([^[\"\`\]]+)")
MOUSE_OUT_PATTERN: Pattern = compile(r"(?<=mouseout=[\"\`])([^[\"\`\]]+)")
MOUSE_OVER_PATTERN: Pattern = compile(r"(?<=mouseover=[\"\`])([^[\"\`\]]+)")
MOUSE_UP_PATTERN: Pattern = compile(r"(?<=mouseup=[\"\`])([^[\"\`\]]+)")
MOUSE_MOVE_PATTERN: Pattern = compile(r"(?<=mousemove=[\"\`])([^[\"\`\]]+)")



import minify_html
import typing as t
from re import findall

from .regex_patterns import NEW_LINE_REPLACE_PATTERN


def replace_regex(RE_PATTERN: t.Pattern, html: str) -> str:
    regex_occurances: t.List = findall(RE_PATTERN, html)
    replaced_regex_occurances: t.List = []

    for i in regex_occurances:
        minified_js = minify_html.minify(
            i,
            minify_js=True,
        )
        replaced_regex_occurances.append(minified_js)

    for i in replaced_regex_occurances:
        index = replaced_regex_occurances.index(i)
        previous_content = regex_occurances[index]

        html = html.replace(previous_content, i)

    return html



"""
    HTML Minifier functions for CSS-HTML-JS-Minify.
        Source : https://github.com/juancarlospaco/css-html-js-minify/blob/master/css_html_js_minify/html_minifier.py
        License : https://github.com/juancarlospaco/css-html-js-minify/blob/master/LICENSE.gpl.txt
            Rewritten By : Baseplate-Admin (https://github.com/baseplate-admin/)
"""
import re
import typing as t


def condense_html_whitespace(html: str) -> str:
    """Condense HTML, but be safe first if it have textareas or pre tags.

    >>> condense_html_whitespace('<i>  <b>    <a> test </a>    </b> </i><br>')
    '<i><b><a> test </a></b></i><br>'
    """  # first space between tags, then empty new lines and in-between.
    tagsStack: t.List = []

    split: t.List = re.split(
        "(<\\s*pre.*>|<\\s*/\\s*pre\\s*>|<\\s*textarea.*>|<\\s*/\\s*textarea\\s*>)",
        html,
        flags=re.IGNORECASE,
    )

    for i in range(0, len(split)):
        # if we are on a tag
        if (i + 1) % 2 == 0:
            tag: str = rawtag(split[i])

            if tag.startswith("/"):
                if not tagsStack or "/" + tagsStack.pop() != tag:
                    raise Exception("Some tag is not closed properly")

            else:
                tagsStack.append(tag)

            continue

        # else check if we are outside any nested <pre>/<textarea> tag
        if not tagsStack:

            temp: str = re.sub(r">\s+<", "> <", split[i])
            split[i] = re.sub(r"\s{2,}|[\r\n]", " ", temp)

    return "".join(split)


def rawtag(string: str) -> str:
    if re.match("<\\s*pre.*>", string, flags=re.IGNORECASE):
        return "pre"

    if re.match("<\\s*textarea.*>", string, flags=re.IGNORECASE):
        return "txt"

    if re.match("<\\s*/\\s*pre\\s*>", string, flags=re.IGNORECASE):
        return "/pre"

    if re.match("<\\s*/\\s*textarea\\s*>", string, flags=re.IGNORECASE):
        return "/txt"


def condense_style(html: str) -> str:
    """Condense style html tags.

    >>> condense_style('<style type="text/css">*{border:0}</style><p>a b c')
    '<style>*{border:0}</style><p>a b c'
    """  # May look silly but Emmet does this and is wrong.
    return (
        html.replace('<style type="text/css">', "<style>")
        .replace("<style type='text/css'>", "<style>")
        .replace("<style type=text/css>", "<style>")
    )


def condense_script(html: str) -> str:
    """Condense script html tags.

    >>> condense_script('<script type="text/javascript"> </script><p>a b c')
    '<script> </script><p>a b c'
    """  # May look silly but Emmet does this and is wrong.
    return (
        html.replace('<script type="text/javascript">', "<script>")
        .replace("<style type='text/javascript'>", "<script>")
        .replace("<style type=text/javascript>", "<script>")
    )


def clean_unneeded_html_tags(html: str) -> str:
    """Clean unneeded t.Optional html tags."""
    tags_to_remove: t.List = [
        "</area>",
        "</base>",
        "</col>",
        "</colgroup>",
        "</img>",
        "</input>",
        "</link>",
        "</meta>",
    ]

    for i in tags_to_remove:
        html = html.replace(i, "")

    return html


def remove_html_comments(html: str) -> str:
    """Remove all HTML comments, Keep all for Grunt, Grymt and IE.

    >>> _="<!-- build:dev -->a<!-- endbuild -->b<!--[if IE 7]>c<![endif]--> "
    >>> _+= "<!-- kill me please -->keep" ; remove_html_comments(_)
    '<!-- build:dev -->a<!-- endbuild -->b<!--[if IE 7]>c<![endif]--> keep'
    """  # Grunt uses comments to as build arguments, bad practice but still.
    return re.compile(r"<!-- .*? -->", re.I).sub("", html)


def unquote_html_attributes(html: str) -> str:
    """Remove all HTML quotes on attibutes if possible.

    >>> unquote_html_attributes('<img   width="9" height="5" data-foo="0"  >')
    '<img width=9 height=5 data-foo=0 >'
    """  # data-foo=0> might cause errors on IE, we leave 1 space data-foo=0 >
    # cache all regular expressions on variables before we enter the for loop.
    any_tag: re.Pattern = re.compile(r"<\w.*?>", re.I | re.MULTILINE | re.DOTALL)
    space: re.Pattern = re.compile(r" \s+|\s +", re.MULTILINE)
    space1: re.Pattern = re.compile(r"\w\s+\w", re.MULTILINE)
    space2: re.Pattern = re.compile(r'"\s+>', re.MULTILINE)
    space3: re.Pattern = re.compile(r"'\s+>", re.MULTILINE)
    space4: re.Pattern = re.compile(
        "\"\s\s+\w+=\"|'\s\s+\w+='|\"\s\s+\w+=|'\s\s+\w+=", re.MULTILINE
    )
    space6: re.Pattern = re.compile(r"\d\s+>", re.MULTILINE)
    quotes_in_tag: re.Pattern = re.compile('([a-zA-Z]+)="([a-zA-Z0-9-_.]+)"')

    # iterate on a for loop cleaning stuff up on the html markup.
    for tag in iter(any_tag.findall(html)):
        # exceptions of comments and closing tags
        if tag.startswith("<!") or tag.find("</") > -1:
            continue
        original = tag

        # remove white space inside the tag itself
        tag = space2.sub('" >', tag)  # preserve 1 white space is safer
        tag = space3.sub("' >", tag)

        for each in space1.findall(tag) + space6.findall(tag):
            tag = tag.replace(each, space.sub(" ", each))
        for each in space4.findall(tag):
            tag = tag.replace(each, each[0] + " " + each[1:].lstrip())

        # remove quotes on some attributes
        tag = quotes_in_tag.sub(r"\1=\2 ", tag)  # See Bug #28

        if original != tag:  # has the tag been improved ?
            html = html.replace(original, tag)

    return html.strip()


def html_minify(
    html: str,
    STRIP_WHITESPACE_PYTHON_REMOVE_COMMENTS: t.Optional[bool],
    STRIP_WHITESPACE_PYTHON_CONDENSE_STYLE_FROM_HTML: t.Optional[bool],
    STRIP_WHITESPACE_PYTHON_CONDENSE_SCRIPT_FROM_HTML: t.Optional[bool],
    STRIP_WHITESPACE_PYTHON_CLEAN_UNNEEDED_HTML_TAGS: t.Optional[bool],
    STRIP_WHITESPACE_PYTHON_CONDENSE_HTML_WHITESPACE: t.Optional[bool],
    STRIP_WHITESPACE_PYTHON_UNQUOTE_HTML_ATTRIBUTES: t.Optional[bool],
) -> str:
    """Minify HTML main function.

    >>> html_minify(' <p  width="9" height="5"  > <!-- a --> b </p> c <br> ')
    '<p width=9 height=5 > b c <br>'
    """
    if STRIP_WHITESPACE_PYTHON_REMOVE_COMMENTS:
        html = remove_html_comments(html)

    if STRIP_WHITESPACE_PYTHON_CONDENSE_STYLE_FROM_HTML:
        html = condense_style(html)

    if STRIP_WHITESPACE_PYTHON_CONDENSE_SCRIPT_FROM_HTML:
        html = condense_script(html)

    if STRIP_WHITESPACE_PYTHON_CLEAN_UNNEEDED_HTML_TAGS:
        html = clean_unneeded_html_tags(html)

    if STRIP_WHITESPACE_PYTHON_CONDENSE_HTML_WHITESPACE:
        html = condense_html_whitespace(html)

    if STRIP_WHITESPACE_PYTHON_UNQUOTE_HTML_ATTRIBUTES:
        html = unquote_html_attributes(html)

    return html.strip()



"""
This module adds line break to html_files

"""
from ..functions.regex import replace_regex


def add_line_break(html: str, flavor: str) -> str:
    if flavor.lower() == "plain":
        html = html

    elif flavor.lower() == "alpinejs":
        from ..functions.regex_patterns.alpinejs import (
            # X pattern
            X_INIT_PATTERN,
            X_EFFECT_PATTERN,
            # Mouse Click Pattern
            CLICK_PATTERN,
            CLICK_PREVENT_PATTERN,
            CLICK_STOP_PATTERN,
            CLICK_OUTSIDE_PATTERN,
            CLICK_WINDOW_PATTERN,
            CLICK_ONCE_PATTERN,
            CLICK_DEBOUNCE_PATTERN,
            CLICK_THROTTLE_PATTERN,
            CLICK_SELF_PATTERN,
            CLICK_CAMEL_PATTERN,
            CLICK_DOT_PATTERN,
            CLICK_PASSIVE_PATTERN,
            # Mouse movement Pattern
            MOUSE_ENTER_PATTERN,
            MOUSE_LEAVE_PATTERN,
            MOUSE_OUT_PATTERN,
            MOUSE_OVER_PATTERN,
            MOUSE_UP_PATTERN,
            MOUSE_MOVE_PATTERN,
        )

        html = replace_regex(X_INIT_PATTERN, html)
        html = replace_regex(X_EFFECT_PATTERN, html)

        # Mouse Click Pattern
        html = replace_regex(CLICK_PATTERN, html)
        html = replace_regex(CLICK_PREVENT_PATTERN, html)
        html = replace_regex(CLICK_STOP_PATTERN, html)
        html = replace_regex(CLICK_OUTSIDE_PATTERN, html)
        html = replace_regex(CLICK_WINDOW_PATTERN, html)
        html = replace_regex(CLICK_ONCE_PATTERN, html)
        html = replace_regex(CLICK_DEBOUNCE_PATTERN, html)
        html = replace_regex(CLICK_THROTTLE_PATTERN, html)
        html = replace_regex(CLICK_SELF_PATTERN, html)
        html = replace_regex(CLICK_CAMEL_PATTERN, html)
        html = replace_regex(CLICK_DOT_PATTERN, html)
        html = replace_regex(CLICK_PASSIVE_PATTERN, html)

        # Mouse movement Pattern
        html = replace_regex(MOUSE_ENTER_PATTERN, html)
        html = replace_regex(MOUSE_LEAVE_PATTERN, html)
        html = replace_regex(MOUSE_OUT_PATTERN, html)
        html = replace_regex(MOUSE_OVER_PATTERN, html)
        html = replace_regex(MOUSE_UP_PATTERN, html)
        html = replace_regex(MOUSE_MOVE_PATTERN, html)

    elif flavor.lower() == str("petitevue"):
        # We'll implement it later.
        pass

    else:
        raise ValueError(
            f"""Error in python_strip_whitespace.html.line_break.
                    Current Flavor : { flavor }
                    It must be one of these :
                        |>  str("plain")
                        |>  str("alpinejs")
                        |>  str("petitevue")
                    
                    Please change the value when calling add_line_break()"""
        )
    return html



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['python_strip_whitespace',
 'python_strip_whitespace.functions',
 'python_strip_whitespace.functions.regex_patterns',
 'python_strip_whitespace.html']

package_data = \
{'': ['*']}

install_requires = \
['minify-html>=0.10.8,<0.11.0']

setup_kwargs = {
    'name': 'python-strip-whitespace',
    'version': '0.1.1',
    'description': 'A powerful HTML whitespace remover for python',
    'long_description': 'HTML Whitespace remover for Python\n==================================\n|Pepy.tech Badge| |PyPi Version Badge| |Python Versions Badge| |License Badge| |Code Style| |Lines of Code Badge|\n\n.. |Pepy.tech Badge| image:: https://static.pepy.tech/personalized-badge/python-strip-whitespace?period=week&units=international_system&left_color=grey&right_color=orange&left_text=Downloads\n   :target: https://pepy.tech/project/python-strip-whitespace\n\n.. |PyPi Version Badge| image:: https://badge.fury.io/py/python-strip-whitespace.svg\n    :target: https://badge.fury.io/py/python-strip-whitespace\n\n.. |Python Versions Badge| image:: https://img.shields.io/pypi/pyversions/python-strip-whitespace\n    :alt: PyPI - Python Version\n    :target: https://github.com/baseplate-admin/python_strip_whitespace/blob/main/setup.py\n\n.. |License Badge| image:: https://img.shields.io/pypi/l/python-strip-whitespace\n   :alt: PyPI - License\n   :target: https://github.com/baseplate-admin/python_strip_whitespace/blob/main/LICENSE\n   \n.. |Code Style| image:: https://img.shields.io/badge/code%20style-black-000000.svg\n   :alt: Code Style\n   \n.. |Lines of Code Badge| image:: https://tokei.rs/b1/github/baseplate-admin/python_strip_whitespace\n   :alt: Lines of Code\n   :target: https://github.com/baseplate-admin/python_strip_whitespace\n   \nIntroduction :\n--------------\nA powerful tool to optimize HTML\n\nWhy use "python_stip_whitespace" ?\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n*   Adds line break to InlineJS.\n*   It can automagically minify inline CSS, JS.\n*   Removes <!--prettier-ignore--> from HTML.\n*   It speeds up website by reducing the HTML size.\n*   It compiles regex at runtime. So it\'s blazing fast.\n*   Its mostly based on C ( gzip ) and Rust ( `minify-html <https://pypi.org/project/minify-html/>`__  ) libraries.\n*   Significantly lower bytes transferred when working with frameworks like AlpineJs ( Almost fully working & Please open a issue in the `Issue Tracker <https://github.com/baseplate-admin/python_strip_whitespace/issues>`__ if you encounter any bug ) & Petite Vue.\n*   Is very customizable. ( You can configure lower level `minify-html <https://github.com/wilsonzlin/minify-html/blob/master/python/src/lib.template.rs/>`_ rust bindings and also the lower level `python <https://github.com/juancarlospaco/css-html-js-minify/blob/master/css_html_js_minify/html_minifier.py/>`__ bindings when calling minify  )\n\n\nWhy shouldn\'t you use python_stip_whitespace ?\n~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n*   Adds unnecessary \';;\' in HTML. ( If you know any regex to fix this please put a pull request )\n\n\nRequirements :\n--------------\n\n*    `minify-html <https://github.com/wilsonzlin/minify-html>`__\n*    Python 3 ( Should work with all version? )\n*    `Brotli <https://pypi.org/project/Brotli/>`_ ( or `BrotliPy <https://pypi.org/project/brotlipy/>`_ ) | ( Optional )\n*    `ZSTD <https://pypi.org/project/zstandard/>`_ ( Optional ) ``Isn\'t supperted by modern browsers``\n\nUsed Internally by :\n====================\n*     `django-strip-whitespace <https://github.com/baseplate-admin/django_strip_whitespace>`_ \n*     `flask-strip-whitespace <https://github.com/baseplate-admin/flask_strip_whitespace>`_ \n*     `fastapi-strip-whitespace <https://github.com/baseplate-admin/fastapi_strip_whitespace>`_ ( Doesn\'t exist  😛 )\n\nContributing :\n==============\nIf you like this project add a star. \nIf you have problems or suggestions please put them in the `Issue Tracker <https://github.com/baseplate-admin/python_strip_whitespace/issues>`__.\nIf you like to add features. Fork this repo and submit a Pull Request. 😛\n\nRoadmap :\n=========\nYou tell me. If i have free time, I will implement it.\n',
    'author': 'baseplate-admin',
    'author_email': '61817579+baseplate-admin@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
